/*
 * Copyright 2002, 2014 IBM Corp.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.  
 *
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
*/
package clenotes

import java.io.BufferedReader
import java.io.File
import java.io.FileReader
import java.util.List
import java.util.Map
import java.util.Vector

class Configuration {
	public val static PROGRAM_NAME = "Command Line Email Client for IBM Notes"
	public val static VERSION = "5.0.5"

	public val static HEADER = '''PROGRAM_NAME vVERSION
Copyright (C) 2002, 2014 by IBM Corporation.
Licensed under the Apache License v2.0.
'''
	public val static MAILER = '''PROGRAM_NAME vVERSION'''
	public val static LINE_SEPARATOR = System::getProperty("line.separator")

	static var List<Option> globalOptions = new Vector<Option>()
	public var static Map<String, Option> globalOptionMap = newLinkedHashMap()

	var static List<String> options = new Vector<String>()
	public var static commandMap = newLinkedHashMap()
	var static List<String> commands = new Vector<String>()
	public var static configMap = newLinkedHashMap()

	var static LOG_ENABLED = false

	var static logParsing = false

	def static load() {
		val BufferedReader br = new BufferedReader(new FileReader(new File('config/clenotes.cfg')))

		//load config file
		var String line

		while ((line = br.readLine()) != null) {
			switch line {
				case line.startsWith("#") || line.empty: {
					//ignore comments and empty lines
				}
				case line.equals("[options]"):
					ConfigurationSection::setOption
				case line.equals("[commands]"):
					ConfigurationSection::setCommand
				case line.equals("[configuration]"):
					ConfigurationSection::setConfiguration
				case ConfigurationSection::option:
					options.add(line)
				case ConfigurationSection::command:
					commands.add(line)
				case ConfigurationSection::configuration: {
					if (line.contains("write_log")) {
						if (LOG_ENABLED==false && line.toLowerCase().contains("true")) {
							LOG_ENABLED = true;
						}
					}
					val configEntry = line.split(":")
					configMap.put(configEntry.get(0).trim(), configEntry.get(1).trim())

				}
			}
		}
		br.close()
	}

	def static printConfigFileEntries() {
		if (logParsing) {
			Logger::log("Options: " + globalOptions)
			Logger::log("Commands: " + commandMap)
			Logger::log("Configuration: " + configMap)

		}

	}

	def static parse() {

		//parse config file entries
		for (option : options) {
			val firstIndex = option.indexOf(":")
			var name = option.substring(0, firstIndex)
			var desc = option.substring(firstIndex + 1)
			name = name.replace("--", "").trim()
			var mandatoryValue = false;
			if (name.contains("!")) {
				mandatoryValue = true
				name = name.replace("!", "")
			}
			desc = removeFunctionName(desc)
			var Option opt = new Option()

			opt.setName(name)
			opt.setDescription(desc)
			opt.setMandatoryValue(mandatoryValue)

			globalOptions.add(opt)
			globalOptionMap.put(opt.name, opt);

		}

		for (command : commands) {
			val commandName = command.substring(0, command.indexOf("."))
			if (command.contains(".cmd")) {

				//extract command name
				var commandDescription = command.substring(command.indexOf(":") + 1)
				commandDescription = removeFunctionName(commandDescription)
				var cmd = new Command()
				cmd.setName(commandName)
				cmd.setDescription(commandDescription)

				//commandName, commandDescription, new Vector<Option>())
				commandMap.put(commandName, cmd)
			} else {
				var commandObj = commandMap.get(commandName) as Command;
				if (command.contains("--")) {

					//command option
					var optionString = command.substring(command.indexOf(".") + 1)
					if(logParsing) Logger::log("Option in config: " + optionString)
					val cInd = optionString.indexOf(":")
					var optionName = optionString.substring(0, cInd)
					val mandatoryValue = optionName.contains("!")
					optionName = optionName.replace("--", "").replace("!", "").trim()
					if(logParsing) Logger::log("Option in config: " + optionName)
					var optionDesc = optionString.substring(cInd + 1).trim()
					if(logParsing) Logger::log("Option desc config: " + optionDesc)
					var opt = new Option()
					opt.setMandatoryValue(mandatoryValue)
					opt.setName(optionName)
					opt.setDescription(optionDesc)

					var cmdOpts = commandObj.optionMap
					cmdOpts.put(opt.name, opt)
				}
			}

		}

		if (logEnabled) {
			for (commandName : commandMap.keySet) {
				if(logParsing) Logger::log("Command object: " + commandMap.get(commandName))

			}

		}

	}

	private def static removeFunctionName(String _desc) {
		var desc = _desc
		desc = desc.trim()
		val ind = desc.indexOf("|")
		if (ind > -1) {
			desc = desc.substring(0, ind)
		}
		return desc

	}

	def static setLogEnabled(boolean enabled) {
		LOG_ENABLED = enabled

	}

	def static isLogEnabled() {
		return LOG_ENABLED
	}

	def static getConfig(String configName) {
		return configMap.get(configName)
	}

}
